import setuptools
from setuptools import setup

setup(
    name='{{name}}',
    version='1.0.0',
    author='easyDataverse',
    license='MIT License',
    packages=setuptools.find_packages(),
    install_requires=[
        {%- for req in requirements %}
        '{{req}}',
        {%- endfor %}
    ]
)